#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-06-16
#
# rewrite of checkpoint_ha_status
#
#

# sample snmp walk
# .1.3.6.1.4.1.2620.1.5.2.0 1
# .1.3.6.1.4.1.2620.1.5.3.0 6
# .1.3.6.1.4.1.2620.1.5.4.0 0
# .1.3.6.1.4.1.2620.1.5.5.0 yes
# .1.3.6.1.4.1.2620.1.5.6.0 active
# .1.3.6.1.4.1.2620.1.5.7.0 OK
# .1.3.6.1.4.1.2620.1.5.101.0 0
# .1.3.6.1.4.1.2620.1.5.103.0

from typing import Optional
from dataclasses import dataclass
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    SNMPTree,
    startswith,
    State,
    all_of,
    any_of,
    equals,
)


@dataclass
class CheckPointHa:
    installed: int
    major: int
    minor: int
    started: str
    state: str
    block_state: str
    stat_code: int
    stat_long: str


def parse_checkpoint_ha_status(string_table: StringTable) -> Optional[CheckPointHa]:
    try:
        installed, major, minor, started, state, block_state, stat_code, stat_long = string_table[0]
    except IndexError:
        return

    if installed != "0":
        return CheckPointHa(
            installed=int(installed),
            major=int(major),
            minor=int(minor),
            state=state.strip(),
            started=started,
            block_state=block_state,
            stat_code=int(stat_code),
            stat_long=stat_long.strip()
        )


def discovery_checkpoint_ha_status(section: CheckPointHa) -> DiscoveryResult:
    yield Service()


def check_checkpoint_ha_status(params, section: CheckPointHa) -> CheckResult:
    if section.installed == 0:
        yield Result(state=State(params['state_not_installed']), summary='H/A not installed')    
    else:
        yield Result(state=State.OK, summary=f'Installed v{section.major}.{section.minor}')

        for val, infotext, ok_vals, warn_vals, warn_status in [
            (section.started, "Started", ["yes"], None, params['state_not_started']),
            (section.state, "Status", [params['expected_ha_state']], None, params['state_if_not_expected']),
            (section.block_state, "Blocking", ["ok"], ["initializing"], params['state_blocking']),
        ]:
            if ok_vals is None or val.lower() in ok_vals:
                status = 0
            elif warn_vals is not None and val.lower() in warn_vals:
                status = warn_status
            else:
                status = warn_status

            yield Result(state=State(status), summary=f'{infotext}: {val}')
            
        if section.stat_code != 0:
            yield Result(state=State(params['state_problem']), summary=f'Problem: {section.stat_long}')


register.snmp_section(
    name='checkpoint_ha_status',
    parse_function=parse_checkpoint_ha_status,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.5',  # CHECKPOINT-MIB::ha
        oids=[
            '2',  # haInstalled
            '3',  # haVerMajor
            '4',  # haVerMinor
            '5',  # haStarted
            '6',  # haState
            '7',  # haBlockState
            '101',  # haStatCode
            '103',  # haStatLong
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_ha_status',
    service_name='HA Status',
    discovery_function=discovery_checkpoint_ha_status,
    check_function=check_checkpoint_ha_status,
    check_default_parameters={
        'state_not_installed': 2,
        'expected_ha_state': 'active',
        'state_if_not_expected': 1,  # WARN
        'state_not_started': 2,
        'state_blocking': 2,
        'state_problem': 2,
    },
    check_ruleset_name='checkpoint_ha_status',
)
